/*
 * Plasma applet to display indicators from libindicate
 *
 * Copyright 2009 Canonical Ltd.
 *
 * Authors:
 * - Aurélien Gâteau <aurelien.gateau@canonical.com>
 *
 * License: GPL v3
 */
#ifndef FILTERMODELTEST_H
#define FILTERMODELTEST_H

// Qt
#include <QObject>

// libindicate-qt
#include <qindicateserver.h>

// Local
#include <listenermodel.h>

/**
 * Test the listener model
 */
class FilterModelTest : public QObject
{
Q_OBJECT
private Q_SLOTS:
    void init();
    void cleanup();
    void testFilter();

private:
    QIndicate::Listener* mListener;
    QIndicate::Server* mServer;
};

#endif /* FILTERMODELTEST_H */
