/*
 * Plasma applet to display indicators from libindicate
 *
 * Copyright 2009 Canonical Ltd.
 *
 * Authors:
 * - Aurélien Gâteau <aurelien.gateau@canonical.com>
 *
 * License: GPL v3
 */
#ifndef TIMEUTILSTEST_H
#define TIMEUTILSTEST_H

// Qt
#include <QObject>

/**
 * Test the TimeUtils format code
 */
class TimeUtilsTest : public QObject
{
Q_OBJECT
private Q_SLOTS:
    void testFormatDateTime();
    void testFormatDateTime_data();

    void testShortDateFormat();
};

#endif /* TIMEUTILSTEST_H */
